@@ -19,7 +19,7 @@ from pathlib import Path
 from typing import List, Dict, Tuple, Union, Optional
 
 
-VERSION = "0.2.3"
+VERSION = "0.2.5"
 
 GIT_REPOSITORY = "https://github.com/csound-plugins/risset-data"
 
@@ -107,6 +107,36 @@ class PluginDefinitionError(Exception): pass
 class InternalError(Exception): pass
 
 
+def _csound_version() -> Tuple[int, int]:
+    csound_bin = _get_binary("csound")
+    if not csound_bin:
+        raise OSError("csound binary not found")
+    proc = subprocess.Popen([csound_bin, "--version"], stderr=subprocess.PIPE)
+    proc.wait()
+    out = proc.stderr.read().decode('ascii')
+    for line in out.splitlines():
+        if "--Csound version" not in line:
+            continue
+        parts = line.split()
+        versionstr = parts[2]
+        return tuple(int(i) for i in versionstr.split("."))
+    raise ValueError("Could not find a version number in the output")
+
+
+def _get_csound_opcodes() -> List[str]:
+    csound_bin = _get_binary("csound")
+    proc = subprocess.Popen([csound_bin, "-z1"], stderr=subprocess.PIPE)
+    txt = proc.stderr.read().decode('ascii')
+    opcodes = []
+    for line in txt.splitlines():
+        line = line.strip()
+        if not line:
+            continue
+        parts = line.split()
+        opcodes.append(parts[0])
+    return opcodes
+
+
 def _plugin_extension() -> str:
     return {
         'linux': '.so',
@@ -265,7 +295,7 @@ def _find_opcodes_dir(possible_dirs) -> Optional[Path]:
     the system plugins are installed
     """
     ext = _plugin_extension()
-    debug("Finding opcodes dir: ")
+    debug("> Searching opcodes dir: ")
 
     if sys.platform == "win32":
         portaudio_dll = "rtpa.dll"
@@ -273,19 +303,21 @@ def _find_opcodes_dir(possible_dirs) -> Optional[Path]:
         portaudio_dll = "librtpa" + ext
 
     for d in possible_dirs:
-        debug("   > looking at ", d)
+        debug("  >> looking at ", d)
         d = _normalize_path(d)
         path = Path(d)
         if not path.is_dir() or not path.exists():
+            debug("  >>> path does not exist...")
             continue
         plugins = list(path.glob("*" + ext))
         if not plugins:
-            debug(f"Path {d} exists, but has no plugins, skipping")
+            debug(f"  >>> path {d} exists, but has no plugins, skipping")
             continue
         if any(plugin for plugin in plugins if portaudio_dll == plugin.name):
+            debug("  >>> Found!")
             return path
-        errormsg(f"Found plugins dir {d}, but it does not seem to be the systems"
-                 f" plugin path ({portaudio_dll} should be present but was not found)")
+        debug(f"Found plugins dir {d}, but it does not seem to be the systems plugin path"
+              f"  ({portaudio_dll} was not found there)")
     return None
 
 
@@ -443,7 +475,7 @@ def plugin_definition_from_file(filepath: str,
 
     assert path.suffix == ".json", "Plugin definition file should be a .json file"
 
-    debug(f"Parsing {path}")
+    debug("Parsing manifest:", path)
     d = json.load(open(path))
     try:
         plugin = _plugin_from_dict(d, manifest_path=path)
@@ -468,7 +500,7 @@ def load_text(file_or_url:str) -> str:
         return _load_url(file_or_url)
     assert os.path.exists(file_or_url)
     text = open(file_or_url).read()
-    debug(f"load_text: {text}")
+    # debug(f"load_text: \n{text}")
     if not text:
         debug("load_text: empty file")
     return text
@@ -491,10 +523,11 @@ class PluginsIndex:
         self.csoundlib = "CsoundLib64"
         self.apiversion = "6.0"
         self.system_plugins_path = self.get_system_plugins_path()
-        debug(f"System plugins path: {self.system_plugins_path}")
-        self.user_plugins_path = self.get_user_plugins_path()
         if self.system_plugins_path is None:
-            errormsg("Could not find the system plugin folder")
+            debug("Could not find the system plugins folder")
+        else:
+            debug(f"System plugins path: {self.system_plugins_path}")
+        self.user_plugins_path = self.get_user_plugins_path()
 
     def update_git_repository(self) -> None:
         _git_update(self.git_repo)
@@ -561,9 +594,9 @@ class PluginsIndex:
                 _(f"    export OPCODE6DIR64=\"{systempath}:{userpath}\"\n")
         return lines
 
-    def get_system_plugins_path(self) -> Optional[Path]:
+    def get_default_system_plugins_path(self) -> List[str]:
         if self.platform == 'linux':
-            possible_dirs = ["/usr/local/lib/csound/plugins64-6.0"]
+            possible_dirs = ["/usr/local/lib/csound/plugins64-6.0", "/usr/lib/csound/plugins64-6.0"]
         elif self.platform == 'macos':
             # The path based on ~ is used when csound is compiled from source.
             # We give that priority since if a user is doing that, it is probably someone who knows
@@ -575,7 +608,19 @@ class PluginsIndex:
         elif self.platform == "windows":
             possible_dirs = ["C:\\Program Files\\Csound6_x64\\plugins64"]
         else:
-            return None
+            raise PlatformNotSupportedError(f"Platform {self.platform} not supported")
+        return possible_dirs
+
+    def get_system_plugins_path(self) -> Optional[Path]:
+        """
+        Get the path were system plugins are installed.
+        """
+        # first check if the user has set OPCODE6DIR64
+        opcode6dir64 = os.getenv("OPCODE6DIR64")
+        if opcode6dir64:
+            possible_dirs = opcode6dir64.split(_get_path_separator())
+        else:
+            possible_dirs = self.get_default_system_plugins_path()
 
         out = _find_opcodes_dir(possible_dirs)
         if not out:
@@ -749,9 +794,10 @@ class PluginsIndex:
         """
         return _data_dir_for_platform() / "risset"
 
-    def check_plugin_installed(self, plugin:Plugin) -> bool:
-        # TODO
-        return True
+    def is_plugin_installed(self, plugin:Plugin) -> bool:
+        test = plugin.opcodes[0]
+        opcodes = _get_csound_opcodes()
+        return test in opcodes
 
     def get_plugin_dll(self, plugin:Plugin) -> Union[Path, ErrorMsg]:
         """
@@ -805,7 +851,12 @@ class PluginsIndex:
         if not (install_path / plugin_dll).exists():
             return ErrorMsg(f"Installation of plugin {plugin.name} failed")
 
-        # Installation succeeded, install manifest
+        # installation succeeded, check that it works
+        if not self.is_plugin_installed(plugin):
+            opcode = plugin.opcodes[0]
+            return ErrorMsg(f"Plugin binary was installed, but opcode {opcode} not present")
+
+        # install manifest
         manifests_path = self.get_installed_manifests_path()
         if not manifests_path.exists():
             manifests_path.mkdir(parents=True)
@@ -1166,7 +1217,7 @@ def cmd_install(plugins_index:PluginsIndex, args) -> bool:
                 errors.append(f"Installed version of plugin {plugin} is up-to-date")
                 errormsg(errors[-1])
                 continue
-            debug(f"Updating plugin {args.plugin}: "
+            print(f"Updating plugin {plugin}: "
                   f"{current_version} -> {plugin_definition.version}")
         error = plugins_index.install_plugin(plugin_definition, user=args.user)
         if error:
@@ -1214,7 +1265,7 @@ def cmd_man(plugins_index:PluginsIndex, args) -> bool:
         debug("man: no opcodes found")
         return False
     for opcode in opcodes:
-        debug("man: processing opcode ", opcode)
+        debug("man: processing opcode ", opcode.name)
         path = plugins_index.find_manpage(opcode.plugin, opcode.name, markdown=args.markdown)
         if not path:
             errormsg(f"No manpage for opcode {opcode.name}")
@@ -1247,6 +1298,9 @@ def main():
         errormsg("git command not found. Check that git is installed and in the PATH")
         sys.exit(-1)
 
+    csound_version = _csound_version()
+    debug(f"Csound version: {csound_version}")
+
     # Main parser
     parser = argparse.ArgumentParser()
     add_flag(parser, "--debug", help="Print debug information")
